# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["MachineLearnParams"]


class MachineLearnParams(TypedDict, total=False):
    message: Required[Union[Iterable[Dict[str, object]], Dict[str, object], str]]
    """Message content to learn from"""

    user_id: Required[str]
    """Unique identifier for the user"""

    datetime_input: Optional[str]
    """ISO format datetime string for the message timestamp"""

    debug: Optional[bool]
    """Whether to process synchronously for debugging"""

    session_id: Optional[str]
    """Optional session identifier for conversation context"""

    speaker: str
    """Speaker of the message"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["InferenceProcessParams"]


class InferenceProcessParams(TypedDict, total=False):
    messages: Required[Iterable[Dict[str, object]]]
    """List of conversation messages to process"""

    session_id: Required[str]
    """Session identifier for conversation continuity"""

    user_id: Required[str]
    """Unique identifier for the user"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .data_ingest_params import DataIngestParams as DataIngestParams
from .data_ingest_response import DataIngestResponse as DataIngestResponse
from .machine_learn_params import MachineLearnParams as MachineLearnParams
from .machine_query_params import MachineQueryParams as MachineQueryParams
from .api_key_create_params import APIKeyCreateParams as APIKeyCreateParams
from .api_key_list_response import APIKeyListResponse as APIKeyListResponse
from .machine_learn_response import MachineLearnResponse as MachineLearnResponse
from .machine_query_response import MachineQueryResponse as MachineQueryResponse
from .api_key_create_response import APIKeyCreateResponse as APIKeyCreateResponse
from .api_key_revoke_response import APIKeyRevokeResponse as APIKeyRevokeResponse
from .inference_process_params import InferenceProcessParams as InferenceProcessParams
from .user_create_or_get_params import UserCreateOrGetParams as UserCreateOrGetParams
from .inference_process_response import InferenceProcessResponse as InferenceProcessResponse
from .user_create_or_get_response import UserCreateOrGetResponse as UserCreateOrGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["InferenceProcessResponse"]


class InferenceProcessResponse(BaseModel):
    content: str
    """Generated response content"""

    messages: List[Dict[str, object]]
    """Processed messages with memory injections"""

    session_id: str
    """Session identifier"""

    success: Optional[bool] = None
    """Whether the request was processed successfully"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable

import httpx

from ..types import inference_process_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.inference_process_response import InferenceProcessResponse

__all__ = ["InferenceResource", "AsyncInferenceResource"]


class InferenceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return InferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return InferenceResourceWithStreamingResponse(self)

    def process(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        session_id: str,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InferenceProcessResponse:
        """
        Process user messages through the UPL inference engine with
        memory/preference/identity integration.

            This endpoint:
            - Validates authentication tokens
            - Processes messages with memory/preference/identity injection
            - Maintains conversation state

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          messages: List of conversation messages to process

          session_id: Session identifier for conversation continuity

          user_id: Unique identifier for the user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/inference",
            body=maybe_transform(
                {
                    "messages": messages,
                    "session_id": session_id,
                    "user_id": user_id,
                },
                inference_process_params.InferenceProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceProcessResponse,
        )


class AsyncInferenceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncInferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return AsyncInferenceResourceWithStreamingResponse(self)

    async def process(
        self,
        *,
        messages: Iterable[Dict[str, object]],
        session_id: str,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InferenceProcessResponse:
        """
        Process user messages through the UPL inference engine with
        memory/preference/identity integration.

            This endpoint:
            - Validates authentication tokens
            - Processes messages with memory/preference/identity injection
            - Maintains conversation state

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          messages: List of conversation messages to process

          session_id: Session identifier for conversation continuity

          user_id: Unique identifier for the user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/inference",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "session_id": session_id,
                    "user_id": user_id,
                },
                inference_process_params.InferenceProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceProcessResponse,
        )


class InferenceResourceWithRawResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.process = to_raw_response_wrapper(
            inference.process,
        )


class AsyncInferenceResourceWithRawResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.process = async_to_raw_response_wrapper(
            inference.process,
        )


class InferenceResourceWithStreamingResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.process = to_streamed_response_wrapper(
            inference.process,
        )


class AsyncInferenceResourceWithStreamingResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.process = async_to_streamed_response_wrapper(
            inference.process,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional

import httpx

from .job import (
    JobResource,
    AsyncJobResource,
    JobResourceWithRawResponse,
    AsyncJobResourceWithRawResponse,
    JobResourceWithStreamingResponse,
    AsyncJobResourceWithStreamingResponse,
)
from ...types import data_ingest_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.data_ingest_response import DataIngestResponse

__all__ = ["DataResource", "AsyncDataResource"]


class DataResource(SyncAPIResource):
    @cached_property
    def job(self) -> JobResource:
        return JobResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return DataResourceWithStreamingResponse(self)

    def ingest(
        self,
        *,
        content_type: str,
        payload: Union[str, Dict[str, object]],
        user_id: str,
        filename: Optional[str] | NotGiven = NOT_GIVEN,
        timestamp_override: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataIngestResponse:
        """
        Generic data ingestion endpoint that accepts various content types and v1 them
        for processing.

            This endpoint follows the "thin edge, fat worker" pattern:
            - Validates authentication and request data
            - Uploads content to S3 storage
            - Queues processing job for async workers
            - Returns immediately with job tracking information

            **Payload Formats:**
            - Raw string content
            - JSON object with structured data
            - Base64 encoded binary data (for file content)

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          content_type: MIME-ish content type string (e.g., 'email', 'text', 'file:text/plain')

          payload: Raw content as string, object, or base64 encoded data

          user_id: User ID to associate the data with

          filename: Filename of the uploaded file

          timestamp_override: ISO-8601 timestamp to preserve original data moment

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/data/ingest",
            body=maybe_transform(
                {
                    "content_type": content_type,
                    "payload": payload,
                    "user_id": user_id,
                    "filename": filename,
                    "timestamp_override": timestamp_override,
                },
                data_ingest_params.DataIngestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataIngestResponse,
        )


class AsyncDataResource(AsyncAPIResource):
    @cached_property
    def job(self) -> AsyncJobResource:
        return AsyncJobResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return AsyncDataResourceWithStreamingResponse(self)

    async def ingest(
        self,
        *,
        content_type: str,
        payload: Union[str, Dict[str, object]],
        user_id: str,
        filename: Optional[str] | NotGiven = NOT_GIVEN,
        timestamp_override: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataIngestResponse:
        """
        Generic data ingestion endpoint that accepts various content types and v1 them
        for processing.

            This endpoint follows the "thin edge, fat worker" pattern:
            - Validates authentication and request data
            - Uploads content to S3 storage
            - Queues processing job for async workers
            - Returns immediately with job tracking information

            **Payload Formats:**
            - Raw string content
            - JSON object with structured data
            - Base64 encoded binary data (for file content)

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          content_type: MIME-ish content type string (e.g., 'email', 'text', 'file:text/plain')

          payload: Raw content as string, object, or base64 encoded data

          user_id: User ID to associate the data with

          filename: Filename of the uploaded file

          timestamp_override: ISO-8601 timestamp to preserve original data moment

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/data/ingest",
            body=await async_maybe_transform(
                {
                    "content_type": content_type,
                    "payload": payload,
                    "user_id": user_id,
                    "filename": filename,
                    "timestamp_override": timestamp_override,
                },
                data_ingest_params.DataIngestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataIngestResponse,
        )


class DataResourceWithRawResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

        self.ingest = to_raw_response_wrapper(
            data.ingest,
        )

    @cached_property
    def job(self) -> JobResourceWithRawResponse:
        return JobResourceWithRawResponse(self._data.job)


class AsyncDataResourceWithRawResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

        self.ingest = async_to_raw_response_wrapper(
            data.ingest,
        )

    @cached_property
    def job(self) -> AsyncJobResourceWithRawResponse:
        return AsyncJobResourceWithRawResponse(self._data.job)


class DataResourceWithStreamingResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

        self.ingest = to_streamed_response_wrapper(
            data.ingest,
        )

    @cached_property
    def job(self) -> JobResourceWithStreamingResponse:
        return JobResourceWithStreamingResponse(self._data.job)


class AsyncDataResourceWithStreamingResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

        self.ingest = async_to_streamed_response_wrapper(
            data.ingest,
        )

    @cached_property
    def job(self) -> AsyncJobResourceWithStreamingResponse:
        return AsyncJobResourceWithStreamingResponse(self._data.job)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import api_key_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.api_key_list_response import APIKeyListResponse
from ..types.api_key_create_response import APIKeyCreateResponse
from ..types.api_key_revoke_response import APIKeyRevokeResponse

__all__ = ["APIKeysResource", "AsyncAPIKeysResource"]


class APIKeysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> APIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return APIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> APIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return APIKeysResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        label: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyCreateResponse:
        """
        Create a new API key for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Creates a new API key for the same user/organization
            - Returns the new API key (only shown once)
            - Supports optional labeling for key management

            **Authentication**: Requires valid API key in Authorization header

        Args:
          label: Optional label for the API key

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/api-keys/",
            body=maybe_transform({"label": label}, api_key_create_params.APIKeyCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyCreateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyListResponse:
        """
        Retrieve all API keys for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Returns list of all API keys for the user (without the actual key values)
            - Includes metadata like creation time and last usage

            **Authentication**: Requires valid API key in Authorization header
        """
        return self._get(
            "/v1/api-keys/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyListResponse,
        )

    def revoke(
        self,
        api_key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyRevokeResponse:
        """
        Revoke (delete) an API key for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Revokes the specified API key (if it belongs to the user)
            - Returns confirmation of revocation

            **Authentication**: Requires valid API key in Authorization header

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return self._delete(
            f"/v1/api-keys/{api_key_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyRevokeResponse,
        )


class AsyncAPIKeysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAPIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAPIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAPIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return AsyncAPIKeysResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        label: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyCreateResponse:
        """
        Create a new API key for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Creates a new API key for the same user/organization
            - Returns the new API key (only shown once)
            - Supports optional labeling for key management

            **Authentication**: Requires valid API key in Authorization header

        Args:
          label: Optional label for the API key

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/api-keys/",
            body=await async_maybe_transform({"label": label}, api_key_create_params.APIKeyCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyCreateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyListResponse:
        """
        Retrieve all API keys for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Returns list of all API keys for the user (without the actual key values)
            - Includes metadata like creation time and last usage

            **Authentication**: Requires valid API key in Authorization header
        """
        return await self._get(
            "/v1/api-keys/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyListResponse,
        )

    async def revoke(
        self,
        api_key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyRevokeResponse:
        """
        Revoke (delete) an API key for the authenticated user.

            This endpoint:
            - Validates the current API key authentication
            - Revokes the specified API key (if it belongs to the user)
            - Returns confirmation of revocation

            **Authentication**: Requires valid API key in Authorization header

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return await self._delete(
            f"/v1/api-keys/{api_key_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyRevokeResponse,
        )


class APIKeysResourceWithRawResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = to_raw_response_wrapper(
            api_keys.create,
        )
        self.list = to_raw_response_wrapper(
            api_keys.list,
        )
        self.revoke = to_raw_response_wrapper(
            api_keys.revoke,
        )


class AsyncAPIKeysResourceWithRawResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = async_to_raw_response_wrapper(
            api_keys.create,
        )
        self.list = async_to_raw_response_wrapper(
            api_keys.list,
        )
        self.revoke = async_to_raw_response_wrapper(
            api_keys.revoke,
        )


class APIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = to_streamed_response_wrapper(
            api_keys.create,
        )
        self.list = to_streamed_response_wrapper(
            api_keys.list,
        )
        self.revoke = to_streamed_response_wrapper(
            api_keys.revoke,
        )


class AsyncAPIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = async_to_streamed_response_wrapper(
            api_keys.create,
        )
        self.list = async_to_streamed_response_wrapper(
            api_keys.list,
        )
        self.revoke = async_to_streamed_response_wrapper(
            api_keys.revoke,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional

import httpx

from ..types import machine_learn_params, machine_query_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.machine_learn_response import MachineLearnResponse
from ..types.machine_query_response import MachineQueryResponse

__all__ = ["MachineResource", "AsyncMachineResource"]


class MachineResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MachineResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return MachineResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MachineResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return MachineResourceWithStreamingResponse(self)

    def learn(
        self,
        *,
        message: Union[Iterable[Dict[str, object]], Dict[str, object], str],
        user_id: str,
        datetime_input: Optional[str] | NotGiven = NOT_GIVEN,
        debug: Optional[bool] | NotGiven = NOT_GIVEN,
        session_id: Optional[str] | NotGiven = NOT_GIVEN,
        speaker: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MachineLearnResponse:
        """
        Process user messages to extract and store episodic memories, preferences, and
        identity attributes.

            This endpoint:
            - Validates authentication tokens
            - Processes conversation messages for memory extraction
            - Queues learning jobs for asynchronous processing (default)
            - Supports synchronous processing in debug mode
            - Maintains conversation state and session continuity

            **Authentication**: Requires valid JWT token in Authorization header

            **Processing Modes**:
            - Normal mode (debug=false): Queues job to SQS for asynchronous processing
            - Debug mode (debug=true): Processes synchronously for immediate results

        Args:
          message: Message content to learn from

          user_id: Unique identifier for the user

          datetime_input: ISO format datetime string for the message timestamp

          debug: Whether to process synchronously for debugging

          session_id: Optional session identifier for conversation context

          speaker: Speaker of the message

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/machine/learn",
            body=maybe_transform(
                {
                    "message": message,
                    "user_id": user_id,
                    "datetime_input": datetime_input,
                    "debug": debug,
                    "session_id": session_id,
                    "speaker": speaker,
                },
                machine_learn_params.MachineLearnParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MachineLearnResponse,
        )

    def query(
        self,
        *,
        question: str,
        user_id: str,
        session_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MachineQueryResponse:
        """
        Query user's stored memories, preferences, and identity attributes based on a
        natural language question.

            This endpoint:
            - Validates authentication tokens
            - Retrieves relevant memories using semantic search
            - Returns structured memory data from all memory types
            - Supports optional session context for conversation continuity

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          question: The question to query against user's memories

          user_id: Unique identifier for the user

          session_id: Optional session identifier for conversation context

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/machine/query",
            body=maybe_transform(
                {
                    "question": question,
                    "user_id": user_id,
                    "session_id": session_id,
                },
                machine_query_params.MachineQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MachineQueryResponse,
        )


class AsyncMachineResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMachineResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMachineResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMachineResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ElicitLabs/modal-python-sdk#with_streaming_response
        """
        return AsyncMachineResourceWithStreamingResponse(self)

    async def learn(
        self,
        *,
        message: Union[Iterable[Dict[str, object]], Dict[str, object], str],
        user_id: str,
        datetime_input: Optional[str] | NotGiven = NOT_GIVEN,
        debug: Optional[bool] | NotGiven = NOT_GIVEN,
        session_id: Optional[str] | NotGiven = NOT_GIVEN,
        speaker: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MachineLearnResponse:
        """
        Process user messages to extract and store episodic memories, preferences, and
        identity attributes.

            This endpoint:
            - Validates authentication tokens
            - Processes conversation messages for memory extraction
            - Queues learning jobs for asynchronous processing (default)
            - Supports synchronous processing in debug mode
            - Maintains conversation state and session continuity

            **Authentication**: Requires valid JWT token in Authorization header

            **Processing Modes**:
            - Normal mode (debug=false): Queues job to SQS for asynchronous processing
            - Debug mode (debug=true): Processes synchronously for immediate results

        Args:
          message: Message content to learn from

          user_id: Unique identifier for the user

          datetime_input: ISO format datetime string for the message timestamp

          debug: Whether to process synchronously for debugging

          session_id: Optional session identifier for conversation context

          speaker: Speaker of the message

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/machine/learn",
            body=await async_maybe_transform(
                {
                    "message": message,
                    "user_id": user_id,
                    "datetime_input": datetime_input,
                    "debug": debug,
                    "session_id": session_id,
                    "speaker": speaker,
                },
                machine_learn_params.MachineLearnParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MachineLearnResponse,
        )

    async def query(
        self,
        *,
        question: str,
        user_id: str,
        session_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MachineQueryResponse:
        """
        Query user's stored memories, preferences, and identity attributes based on a
        natural language question.

            This endpoint:
            - Validates authentication tokens
            - Retrieves relevant memories using semantic search
            - Returns structured memory data from all memory types
            - Supports optional session context for conversation continuity

            **Authentication**: Requires valid JWT token in Authorization header

        Args:
          question: The question to query against user's memories

          user_id: Unique identifier for the user

          session_id: Optional session identifier for conversation context

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/machine/query",
            body=await async_maybe_transform(
                {
                    "question": question,
                    "user_id": user_id,
                    "session_id": session_id,
                },
                machine_query_params.MachineQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MachineQueryResponse,
        )


class MachineResourceWithRawResponse:
    def __init__(self, machine: MachineResource) -> None:
        self._machine = machine

        self.learn = to_raw_response_wrapper(
            machine.learn,
        )
        self.query = to_raw_response_wrapper(
            machine.query,
        )


class AsyncMachineResourceWithRawResponse:
    def __init__(self, machine: AsyncMachineResource) -> None:
        self._machine = machine

        self.learn = async_to_raw_response_wrapper(
            machine.learn,
        )
        self.query = async_to_raw_response_wrapper(
            machine.query,
        )


class MachineResourceWithStreamingResponse:
    def __init__(self, machine: MachineResource) -> None:
        self._machine = machine

        self.learn = to_streamed_response_wrapper(
            machine.learn,
        )
        self.query = to_streamed_response_wrapper(
            machine.query,
        )


class AsyncMachineResourceWithStreamingResponse:
    def __init__(self, machine: AsyncMachineResource) -> None:
        self._machine = machine

        self.learn = async_to_streamed_response_wrapper(
            machine.learn,
        )
        self.query = async_to_streamed_response_wrapper(
            machine.query,
        )
